package ch.zhaw.ads;

import java.util.Scanner;
import java.util.Queue;
import java.util.PriorityQueue;

public class RouteServer implements CommandExecutor {
    Graph<DijkstraNode, Edge> graph;

    @Override
    public String execute(String command) {
        graph = this.readInput(command);
        StringBuffer result = new StringBuffer(100);
        for (Node node : graph.getNodes()) {
            result.append(node.getName() + "\n");
        }
        result.append("\"\n");


        this.dijkstra("Winterthur", "Lugano");
        return result.toString();
    }

    private Graph<DijkstraNode, Edge> readInput(String input) {
        int dist;
        String line, src, dest;
        String[] values;

        Scanner scanner = new Scanner(input);
        Graph<DijkstraNode, Edge> graph = new AdjListGraph<DijkstraNode, Edge>(DijkstraNode.class, Edge.class);
        while (scanner.hasNextLine()) {
            line = scanner.nextLine();

            try {
                values = line.split(" ");
                graph.addEdge(values[0], values[1], Integer.valueOf(values[2]));
                graph.addEdge(values[1], values[0], Integer.valueOf(values[2]));
            } catch (Throwable e) {
                System.err.println("Error.");
                continue;
            }
        }
        scanner.close();
        return graph;
    }

    private void dijkstra(String from, String to) {
        for (DijkstraNode<Edge> node : graph.getNodes()) {
            node.setMark(false);
            node.setDist(Double.POSITIVE_INFINITY);
            node.setPrev(null);
        }

        Queue<DijkstraNode> pq = new PriorityQueue<DijkstraNode>();
        DijkstraNode<Edge> start = graph.findNode(from);
        DijkstraNode<Edge> goal = graph.findNode(to);
        start.setDist(0);
        pq.add(start);
        while (!pq.isEmpty()) {
            DijkstraNode<Edge> current = pq.poll();
            current.setMark(true);
            if (current.equals(goal)) {
                break;
            }
            for (Edge<DijkstraNode> edge : current.getEdges()) {
                DijkstraNode<Edge> sucessor = edge.getDest();
                if (!sucessor.getMark()) {
                    double dist = edge.getWeight() + current.getDist();
                    if (sucessor.getPrev() == null || dist < sucessor.getDist()) {
                        sucessor.setDist(dist);
                        sucessor.setPrev(current);
                        pq.add(sucessor);
                    }
                }
            }
        }

        System.out.println("\n\nShortest path:\n");
        // print the shortest path
        DijkstraNode<Edge> current = goal;
        while (current.getPrev() != null) {
            System.out.println(current.getName());
            current = current.getPrev();
        }
        System.out.println(current.getName());
        System.out.println("\n\n");
    }
}
